'use strict';

var bs58 = require('bs58');

function _interopDefault (e) { return e && e.__esModule ? e : { default: e }; }

var bs58__default = /*#__PURE__*/_interopDefault(bs58);

// src/blockhash.ts
function assertIsBlockhash(putativeBlockhash) {
  try {
    if (
      // Lowest value (32 bytes of zeroes)
      putativeBlockhash.length < 32 || // Highest value (32 bytes of 255)
      putativeBlockhash.length > 44
    ) {
      throw new Error("Expected input string to decode to a byte array of length 32.");
    }
    const bytes = bs58__default.default.decode(putativeBlockhash);
    const numBytes = bytes.byteLength;
    if (numBytes !== 32) {
      throw new Error(`Expected input string to decode to a byte array of length 32. Actual length: ${numBytes}`);
    }
  } catch (e) {
    throw new Error(`\`${putativeBlockhash}\` is not a blockhash`, {
      cause: e
    });
  }
}

// src/params-patcher.ts
function visitNode(value, keyPath, onIntegerOverflow) {
  if (Array.isArray(value)) {
    return value.map(
      (element, ii) => visitNode(element, [...keyPath, ii], onIntegerOverflow)
    );
  } else if (typeof value === "object" && value !== null) {
    const out = {};
    for (const propName in value) {
      if (Object.prototype.hasOwnProperty.call(value, propName)) {
        out[propName] = visitNode(value[propName], [...keyPath, propName], onIntegerOverflow);
      }
    }
    return out;
  } else if (typeof value === "bigint") {
    if (onIntegerOverflow && (value > Number.MAX_SAFE_INTEGER || value < -Number.MAX_SAFE_INTEGER)) {
      onIntegerOverflow(keyPath, value);
    }
    return Number(value);
  } else {
    return value;
  }
}
function patchParamsForSolanaLabsRpc(params, onIntegerOverflow) {
  return visitNode(params, [], onIntegerOverflow);
}

// src/response-patcher-types.ts
var KEYPATH_WILDCARD = {};

// src/response-patcher-allowed-numeric-values.ts
var ALLOWED_NUMERIC_KEYPATHS = {
  getBlockTime: [[]],
  getInflationReward: [[KEYPATH_WILDCARD, "commission"]],
  getRecentPerformanceSamples: [[KEYPATH_WILDCARD, "samplePeriodSecs"]],
  getTokenLargestAccounts: [
    ["value", KEYPATH_WILDCARD, "decimals"],
    ["value", KEYPATH_WILDCARD, "uiAmount"]
  ],
  getTransaction: [
    ["meta", "preTokenBalances", KEYPATH_WILDCARD, "accountIndex"],
    ["meta", "preTokenBalances", KEYPATH_WILDCARD, "uiTokenAmount", "decimals"],
    ["meta", "postTokenBalances", KEYPATH_WILDCARD, "accountIndex"],
    ["meta", "postTokenBalances", KEYPATH_WILDCARD, "uiTokenAmount", "decimals"],
    ["meta", "rewards", KEYPATH_WILDCARD, "commission"],
    ["meta", "innerInstructions", KEYPATH_WILDCARD, "index"],
    ["meta", "innerInstructions", KEYPATH_WILDCARD, "instructions", KEYPATH_WILDCARD, "programIdIndex"],
    ["meta", "innerInstructions", KEYPATH_WILDCARD, "instructions", KEYPATH_WILDCARD, "accounts", KEYPATH_WILDCARD],
    ["transaction", "message", "addressTableLookups", KEYPATH_WILDCARD, "writableIndexes", KEYPATH_WILDCARD],
    ["transaction", "message", "addressTableLookups", KEYPATH_WILDCARD, "readonlyIndexes", KEYPATH_WILDCARD],
    ["transaction", "message", "instructions", KEYPATH_WILDCARD, "programIdIndex"],
    ["transaction", "message", "instructions", KEYPATH_WILDCARD, "accounts", KEYPATH_WILDCARD],
    ["transaction", "message", "header", "numReadonlySignedAccounts"],
    ["transaction", "message", "header", "numReadonlyUnsignedAccounts"],
    ["transaction", "message", "header", "numRequiredSignatures"]
  ],
  getVoteAccounts: [
    ["current", KEYPATH_WILDCARD, "commission"],
    ["delinquent", KEYPATH_WILDCARD, "commission"]
  ]
};

// src/response-patcher.ts
function getNextAllowedKeypaths(keyPaths, property) {
  return keyPaths.filter((keyPath) => keyPath[0] === KEYPATH_WILDCARD && typeof property === "number" || keyPath[0] === property).map((keyPath) => keyPath.slice(1));
}
function visitNode2(value, allowedKeypaths) {
  if (Array.isArray(value)) {
    return value.map((element, ii) => {
      const nextAllowedKeypaths = getNextAllowedKeypaths(allowedKeypaths, ii);
      return visitNode2(element, nextAllowedKeypaths);
    });
  } else if (typeof value === "object" && value !== null) {
    const out = {};
    for (const [propName, innerValue] of Object.entries(value)) {
      const nextAllowedKeypaths = getNextAllowedKeypaths(allowedKeypaths, propName);
      out[propName] = visitNode2(innerValue, nextAllowedKeypaths);
    }
    return out;
  } else if (typeof value === "number" && // The presence of an allowed keypath on the route to this value implies it's allowlisted;
  // Upcast the value to `bigint` unless an allowed keypath is present.
  allowedKeypaths.length === 0) {
    return BigInt(value);
  } else {
    return value;
  }
}
function patchResponseForSolanaLabsRpc(rawResponse, methodName) {
  const allowedKeypaths = methodName ? ALLOWED_NUMERIC_KEYPATHS[methodName] : void 0;
  return visitNode2(rawResponse, allowedKeypaths ?? []);
}

// src/rpc-methods/index.ts
function createSolanaRpcApi(config) {
  return new Proxy({}, {
    defineProperty() {
      return false;
    },
    deleteProperty() {
      return false;
    },
    get(...args) {
      const [_, p] = args;
      const methodName = p.toString();
      return function(...rawParams) {
        const handleIntegerOverflow = config?.onIntegerOverflow;
        const params = patchParamsForSolanaLabsRpc(
          rawParams,
          handleIntegerOverflow ? (keyPath, value) => handleIntegerOverflow(methodName, keyPath, value) : void 0
        );
        return {
          methodName,
          params,
          responseProcessor: (rawResponse) => patchResponseForSolanaLabsRpc(rawResponse, methodName)
        };
      };
    }
  });
}

exports.assertIsBlockhash = assertIsBlockhash;
exports.createSolanaRpcApi = createSolanaRpcApi;
//# sourceMappingURL=out.js.map
//# sourceMappingURL=index.browser.cjs.map